let cheerio = require('cheerio');
let request = require('request');
let requestPromise = require('request-promise-native');
let moment = require('moment');
let Util = require('./util');
moment.locale('pt-br');

let requester = {};

requester.doHsystemRequest = function (companyId, checkIn, checkOut, AmountAdults, AmountChildren, language, childrenAges, cupom) {


    let promise = new Promise(function (resolve, reject) {

        let json =
            {
                "CompanyId": companyId,
                "ArrivalDateString": checkIn,
                "DepartureDateString": checkOut,
                "AmountAdults": AmountAdults,
                "AmountChildren": AmountChildren,
                "ChildrenAges": childrenAges,
                "IsSkipedBooking": false,
                "PromotionalCode": cupom
            };


        console.log("JSON", json);

        request.post({
            url: 'https://hbook.hsystem.com.br/Booking/GetAvailability',
            json: json
        }, function (err, httpResponse, bodymaster) {


            console.log('https://hbook.hsystem.com.br/booking?companyId=' + companyId + '&checkin=&checkin_submit=&checkout=&checkout_submit=&promocode=&language=' + language);

            request.get({url: 'https://hbook.hsystem.com.br/booking?companyId=' + companyId + '&checkin=&checkin_submit=&checkout=&checkout_submit=&promocode=&language=' + language}, function (err, httpResponse, body) {


                if (!err) {

                    let $ = cheerio.load(body);

                    let data = $('#body > section > script:nth-child(35)').text();

                    //  console.log(data);

                    var childrenAngesStr = "";
                    if (childrenAges && childrenAges.length > 0){
                        childrenAges.forEach( item => {
                            childrenAngesStr += '&childrenage='+item.Age;
                        })
                    }

                    var strCupom = "";
                    if (cupom){
                     strCupom = `&promocode=${cupom}`;
                    }

                    let returno = {
                        data: data,
                        body: bodymaster,
                        url: `https://hbook.hsystem.com.br/booking?companyId=${companyId}&checkin=${json.ArrivalDateString}&checkin_submit=${json.ArrivalDateString}&checkout=${json.DepartureDateString}&checkout_submit=${json.DepartureDateString}&adults=${json.AmountAdults}&children=${json.AmountChildren}&language=${language}${childrenAngesStr}${strCupom}`
                    };

                    console.log(returno.url);

                    resolve(returno);
                }
                else {
                    reject(Error("It broke"));
                }


            });

        })


    });


    return promise;
};


requester.getAvailableDates = async function (companyId, checkIn, checkOut, AmountAdults, AmountChildren, childrenAges, cupom) {
    let requestJson = {
        "CompanyId": companyId,
        "ArrivalDateString": checkIn,
        "DepartureDateString": checkOut,
        "AmountAdults": AmountAdults,
        "AmountChildren": AmountChildren,
        "ChildrenAges": childrenAges,
        "IsSkipedBooking": false,
        "PromotionalCode": cupom
    };

    try {
        let response = await requestPromise.post({
            url: 'https://hbook.hsystem.com.br/Booking/GetAvailability',
            json: requestJson
        });
        // console.log(response);
        return Promise.resolve(response);

    } catch (err) {
        console.log("caiu aqui no erro");
        console.error(err);
        return Promise.reject(err);
    }
};


requester.getInventory = async function (companyId, checkIn, checkOut) {
    try {

        let startDate = moment(checkIn, 'DD/MM/YYYY');
        let endDate = moment(checkOut, 'DD/MM/YYYY');
        let now = moment();

        let diffFromStart = startDate.diff(now, 'days');
        //       console.log(diffFromStart);

        let from = now.format('YYYY-MM-DD');

        if (diffFromStart > 14) {
            from = moment(checkIn, 'DD/MM/YYYY').subtract(14, 'days').format('YYYY-MM-DD');
        }
        //console.log(from);

        let daysOfStay = endDate.diff(startDate, 'days');

        //console.log(daysOfStay);
        let monthsAhead = 3;
        let response = await requestPromise.get(`https://hbook.hsystem.com.br/Booking/InventoryPreview?companyId=${companyId}&from=${from}&months=${monthsAhead}`);
        //console.log(response);

        let responseObj = JSON.parse(response).map(r => {
            r.Date = moment(r.Date);
            return r;
        });

        let possibleSuggestions = [];

        responseObj.forEach(dateObj => {
            let possibleDate = dateObj.Date;
            let closedToArrival = dateObj.ClosedToArrival || false;
            let availableOnlyForCheckout = dateObj.AvailableOnlyForCheckout || false;
            if (((possibleDate.day() === startDate.day()) || isNear(possibleDate, startDate)) && !closedToArrival && !availableOnlyForCheckout) {
                possibleSuggestions.push({checkInDate: possibleDate});
            }
        });

        possibleSuggestions.forEach(ps => {
            responseObj.forEach(dateObj => {
                let possibleDate = dateObj.Date;
                let closedToDeparture = dateObj.ClosedToDeparture || false;
                if (possibleDate.diff(ps.checkInDate, 'days') === daysOfStay && !closedToDeparture) {
                    ps.checkOutDate = possibleDate;
                }
            });
        });

        possibleSuggestions = possibleSuggestions.filter(ps => ps.checkOutDate && ps.checkInDate);


        return Promise.resolve(possibleSuggestions);

    } catch (err) {
        console.log("caiu aqui no erro");
        console.error(err);
        return Promise.reject(err);
    }

};


requester.getSuggestionDates = async function (companyId, checkIn, checkOut, AmountAdults, AmountChildren, childrenAges, cupom) {

    try {
        let possibleDates = await requester.getInventory(companyId, checkIn, checkOut);

        let suggestions = [];
        for (let pd of possibleDates) {
            if (suggestions.length < 4) {
                let checkInDate = pd.checkInDate.format('DD/MM/YYYY');
                let checkOutDate = pd.checkOutDate.format('DD/MM/YYYY');
                let availability = await requester.getAvailableDates(companyId, checkInDate, checkOutDate, AmountAdults, AmountChildren, childrenAges, cupom);
                if (availability.Rooms && availability.Rooms.length > 0) {
                    pd.minimumPrice = Util.toReal(availability.Rooms[0].Rates[0].DailyPrice);
                    suggestions.push(pd);
                }
            }
        }


        suggestions = suggestions.map(s => {
            let suggestion = {};
            suggestion.checkIn = s.checkInDate.format('DD/MM/YYYY');
            suggestion.checkOut = s.checkOutDate.format('DD/MM/YYYY');
            let desiredCheckIn = moment(checkIn, 'DD/MM/YYYY');
            suggestion.text = getSuggestionText(s.checkInDate, desiredCheckIn);
            suggestion.period = (s.checkInDate.day() === desiredCheckIn.day() ? 'Mesmos dias da semana' : 'Dias próximos');
            suggestion.minimumPrice = s.minimumPrice;

            return suggestion;
        });

        console.log(suggestions);

        if (suggestions && suggestions.length > 0){
            return Promise.resolve(suggestions);
        }else{
            return Promise.reject(null);
        }

    } catch (err) {
        console.error(err);
        return Promise.reject(err);
    }
};

isNear = function (d1, d2) {
    return Math.abs(d1.diff(d2, 'days')) <= 2;
};


getSuggestionText = function (d1, d2) {
    let text = {};
    let fromTxt = d1.from(d2, true);
    text.amount = fromTxt.split(" ")[0];
    text.param = fromTxt.split(" ")[1];

    if (d1.isBefore(d2)) {
        text.param += " antes";
    } else {
        text.param += " depois";
    }
    return text;
};

// requester.doHsystemRequest("54c9476bec8fc41e68c7a636", "16/06/2018", "17/06/2018", 1, 0, "pt-br", null, null);



//requester.getInventory("53b6e736ec8fc205e869b123", "16/04/2018", "19/04/2018");
// requester.getSuggestionDates("53b6e736ec8fc205e869b123", "17/03/2018", "18/03/2018", 2, 0);


// requester.getSuggestionDates("54c9476bec8fc41e68c7a636", "16/06/2018", "17/06/2018", 1, 0, "pt-br", null, "PRIME");

module.exports = requester;


